@@ -1,6 +1,6 @@
-/*  XNECVIEW - a program for visualizing NEC2 structure and gain files
+/*  XNECVIEW - a program for visualizing NEC2 input and output data
  *
- *  Copyright (C) 1998-1999, P.T. de Boer -- pa3fwm@amsat.org
+ *  Copyright (C) 1998-2000, Pieter-Tjerk de Boer -- pa3fwm@amsat.org
  *
  *  Distributed on the conditions of the GPL: see the files README and
  *  COPYING, which accompany this source file.
@@ -13,7 +13,8 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <values.h>
+#include <limits.h>
+#include <ctype.h>
 
 #include "xnecview.h"
 
@@ -25,6 +26,7 @@
 #define Err_nsurfaces     4
 #define Err_nexcis        5
 #define Err_nloads        6
+#define Err_nnetws        7
 
 
 int nextabs;        /* next absolute segment number to be assigned */
@@ -63,12 +65,12 @@ void out3d(Surface *su,double f)  /* calculates su->pa, with f determining the l
 
 void updateextremes(Point *p)
 {
-   if (p->x>extr) extr=p->x;
-   if (p->y>extr) extr=p->y;
-   if (p->z>extr) extr=p->z;
-   if (-p->x>extr) extr=-p->x;
-   if (-p->y>extr) extr=-p->y;
-   if (-p->z>extr) extr=-p->z;
+   if (p->x>extr_str) extr_str=p->x;
+   if (p->y>extr_str) extr_str=p->y;
+   if (p->z>extr_str) extr_str=p->z;
+   if (-p->x>extr_str) extr_str=-p->x;
+   if (-p->y>extr_str) extr_str=-p->y;
+   if (-p->z>extr_str) extr_str=-p->z;
 }
 
 
@@ -108,38 +110,94 @@ void removecommas(char *p)        /* change commas in string into spaces */
 }
 
 
+int find_segment(int tag, int seg, char *card, Wire **wirep, int *segp)
+{
+   int i;
+   if (tag>0) {                   /* location specified as tag,seg pair */
+      for (i=0;i<numwires;i++) if (wires[i].itg==tag) break;
+      if (i==numwires) { fprintf(stderr,"Unknown tag number on %s card: %i\n",card,tag); return Err_misc; }
+      if (wires[i].ns>0) {
+         if (seg<=0 || seg>wires[i].ns) { fprintf(stderr,"Incorrect segment number on %s card: %i\n",card,seg); return Err_misc; }
+      } else {
+         while (wires[i].ns!=-seg && wires[i].itg==tag) i++;
+         if (wires[i].itg!=tag) { fprintf(stderr,"Incorrect segment number on %s card: %i\n",card,seg); return Err_misc; }
+      }
+      *segp = seg;
+   } else {                       /* location specified as absolute segment number */
+      for (i=0;i<numwires;i++) if (seg>=wires[i].abs && seg<wires[i].abs+wires[i].ns) break;
+      if (i==numwires) { fprintf(stderr,"Incorrect absolute segment number on %s card: %i\n",card,seg); return Err_misc; }
+      *segp = seg-wires[i].abs+1;
+   }
+   *wirep=wires+i;
+   return 0;
+}
+
+
+int read_nec_CM(char *s)     /* CM -> comment card; check whether it contains xnecview options */
+{
+   s+=2;
+   while (*s==' ') s++;
+   if (memcmp(s,"xnecview:",9)==0) process_optionstring(s+9);
+   return 0;
+}
+
 
 int read_nec_EX(char *s)     /* EX -> excitation */
 {
    int type,tag,seg;
-   int i;
+   int err;
 
    removecommas(s);
    if (sscanf(s,"EX%d%d%d",&type,&tag,&seg) != 3) return Err_scan;
    if (type!=0 && type!=5) { fprintf(stderr,"Only voltage sources can be displayed: %s\n",s); return Err_misc; }
 
-   if (tag>0) {                   /* location specified as tag,seg pair */
-      for (i=0;i<numwires;i++) if (wires[i].itg==tag) break;
-      if (i==numwires) { fprintf(stderr,"Unknown tag number on EX card: %i\n",tag); return Err_misc; }
-      if (wires[i].ns>0) {
-         if (seg<=0 || seg>wires[i].ns) { fprintf(stderr,"Incorrect segment number on EX card: %i\n",seg); return Err_misc; }
-      } else {
-         while (wires[i].ns!=-seg && wires[i].itg==tag) i++;
-         if (wires[i].itg!=tag) { fprintf(stderr,"Incorrect segment number on EX card: %i\n",seg); return Err_misc; }
-      }
-   } else {                       /* location specified as absolute segment number */
-      for (i=0;i<numwires;i++) if (seg>=wires[i].abs && seg<wires[i].abs+wires[i].ns) break;
-      if (i==numwires) { fprintf(stderr,"Incorrect absolute segment number on EX card: %i\n",seg); return Err_misc; }
-      seg = seg-wires[i].abs+1;
-   }
    if (numexcis==MAXEXCIS) return Err_nexcis;
-   excis[numexcis].wire=wires+i;
-   excis[numexcis].seg=seg;
+   err=find_segment(tag, seg, "EX", &excis[numexcis].wire, &excis[numexcis].seg);
+   if (err) return err;
    numexcis++;
    return 0;
 }
 
 
+int read_nec_TL(char *s)     /* TL -> transmission line */
+{
+   int tag1,seg1,tag2,seg2;
+   int err;
+   double imp;
+
+   removecommas(s);
+   if (sscanf(s,"TL%d%d%d%d%lg",&tag1,&seg1,&tag2,&seg2,&imp) != 5) return Err_scan;
+
+   if (numnetws==MAXNETWS) return Err_nnetws;
+   err=find_segment(tag1, seg1, "TL", &netws[numnetws].wire1, &netws[numnetws].seg1);
+   if (err) return err;
+   err=find_segment(tag2, seg2, "TL", &netws[numnetws].wire2, &netws[numnetws].seg2);
+   if (err) return err;
+   if (imp<0) netws[numnetws].type = -1; else netws[numnetws].type = 1;
+   numnetws++;
+   return 0;
+}
+
+
+int read_nec_NT(char *s)     /* NT -> network */
+{
+   int tag1,seg1,tag2,seg2;
+   int err;
+
+   removecommas(s);
+   if (sscanf(s,"NT%d%d%d%d",&tag1,&seg1,&tag2,&seg2) != 4) return Err_scan;
+
+   if (numnetws==MAXNETWS) return Err_nnetws;
+   err=find_segment(tag1, seg1, "NT", &netws[numnetws].wire1, &netws[numnetws].seg1);
+   if (err) return err;
+   err=find_segment(tag2, seg2, "NT", &netws[numnetws].wire2, &netws[numnetws].seg2);
+   if (err) return err;
+   netws[numnetws].type = 0;
+   numnetws++;
+   return 0;
+}
+
+
 int addload(Wire *wi,int first,int last)   /* add one load */
 {
    if (numloads==MAXLOADS) return Err_nloads;
@@ -176,7 +234,7 @@ int read_nec_LD(char *s)     /* LD -> loading */
       } else {
          if (segf==0) segf=-1;
          else segf=-segf;
-         if (segl==0) segl=-MAXINT;
+         if (segl==0) segl=-INT_MAX;
          else segl=-segl;
          while (wires[i].ns!=segf && wires[i].itg==tag && i<numwires) i++;
          if (wires[i].itg!=tag || i>=numwires) { fprintf(stderr,"Incorrect first segment number on LD card: %i\n",segf); return Err_misc; }
@@ -184,7 +242,7 @@ int read_nec_LD(char *s)     /* LD -> loading */
             addload(wires+i,1,1);
             i++;
          }
-         if ((wires[i].itg!=tag || i>=numwires) && segl!=-MAXINT) { fprintf(stderr,"Incorrect last segment number on LD card: %i\n",-segl); return Err_misc; }
+         if ((wires[i].itg!=tag || i>=numwires) && segl!=-INT_MAX) { fprintf(stderr,"Incorrect last segment number on LD card: %i\n",-segl); return Err_misc; }
       }
    } else {                       /* location specified as absolute segment number */
       if (segf==0 && segl==0) {
@@ -312,7 +370,7 @@ int read_nec_GM(char *s)    /* GM -> coordinate transformation */
    double rox,roy,roz,xs,ys,zs;
    double mat[9],mmat[9];
    double co,si;
-   Wire *w0,*w1,*wlast;
+   Wire *w00,*w0,*w1,*wlast;
    Surface *s0,*s1;
 
    removecommas(s);
@@ -358,18 +416,23 @@ int read_nec_GM(char *s)    /* GM -> coordinate transformation */
    mat[5] = si*mmat[2] + co*mmat[5];
 
    if (nrpt==0) {
-      w0=w1=wires;
-      i=-1;
+      w00=w1=wires;
    } else {
-      w0=wires;
+      w00=wires;
       w1=wires+numwires;
-      i=0;
    }
-   while (i<nrpt) {
-      wlast=wires+numwires;
-      while (w0<wlast) {
-         if (its==0 || i>0 || w0->itg<=its) {
-            if (numwires>MAXWIRES) return Err_nwires;
+   wlast=wires+numwires;
+   while (w00<wlast) {
+      if (w00->itg>=its) {
+         w0=w00;
+         i=0;
+         do {
+            if (nrpt>0) {
+               if (numwires==MAXWIRES) return Err_nwires;
+               numwires++;
+            } else {
+               w1=w0;
+            }
             pointtransf(&w0->p0,&w1->p0,mat,xs,ys,zs);
             pointtransf(&w0->p1,&w1->p1,mat,xs,ys,zs);
             w1->rad=w0->rad;
@@ -378,13 +441,14 @@ int read_nec_GM(char *s)    /* GM -> coordinate transformation */
             else w1->itg=w0->itg;
             w1->abs=nextabs; nextabs+=w1->ns;
             w1->dispnr=w0->dispnr;
-            w1++; numwires++;
-         }
-         w0++;
+            w0=w1;
+            w1++;
+         } while (++i < nrpt);
       }
-      i++;
+      w00++;
    }
 
+
    if (nrpt==0) {
       s0=s1=surfaces;
       i=numsurfaces;
@@ -542,34 +606,32 @@ void read_nec_input(FILE *f)               /* read the input file, and update th
 {
    char s[100];
    int err;
+   int last_was_NT_or_TL=0;
 
    GeomScale=1.0;
    nextabs=1;
    fgets(s,100,f);
    while (!feof(f) && (s[0]!='E' || s[1]!='N')) {
+      s[0]=toupper(s[0]);
+      s[1]=toupper(s[1]);
       err=0;
       switch (s[0]) {
-         case 'M': 
-            switch (s[1]) {
-               case 'M':
-               case 'E':
-                  printf("%s",s);
-                  break;
-            }
+         case 'C':
+            if (s[1]=='M') err=read_nec_CM(s);
             break;
          case 'E': 
-            if (s[1]=='X') read_nec_EX(s);
+            if (s[1]=='X') err=read_nec_EX(s);
             break;
          case 'G': 
             switch (s[1]) {
                case 'A': err=read_nec_GA(s); break;
+               case 'C': break;  /* just ignore GC cards: they only modify the radius of GW cards, which xnecview doesn't use anyway */
                case 'H': err=read_nec_GH(s); break;
                case 'M': err=read_nec_GM(s); break;
                case 'R': err=read_nec_GR(s); break;
                case 'W': err=read_nec_GW(s); break;
                case 'X': err=read_nec_GX(s); break;
                case 'S': err=read_nec_GS(s); break;
-               case 'C':
                case 'F':
                   fprintf(stderr,"Warning: unhandled cardtype in %s",s);
                   break;
@@ -578,6 +640,21 @@ void read_nec_input(FILE *f)               /* read the input file, and update th
          case 'L': 
             if (s[1]=='D') err=read_nec_LD(s);
             break;
+         case 'M': 
+            switch (s[1]) {
+               case 'M':
+               case 'E':
+                  printf("%s",s);
+                  break;
+            }
+            break;
+         case 'N': 
+            if (s[1]=='T') {
+               if (!last_was_NT_or_TL) numnetws=0;
+               err=read_nec_NT(s);
+               last_was_NT_or_TL=1;
+            }
+            break;
          case 'S':
             switch (s[1]) {
                case 'C': err=read_nec_SC(s); break;
@@ -585,6 +662,13 @@ void read_nec_input(FILE *f)               /* read the input file, and update th
                case 'P': err=read_nec_SP(s); break;
             }
             break;
+         case 'T': 
+            if (s[1]=='L') {
+               if (!last_was_NT_or_TL) numnetws=0;
+               err=read_nec_TL(s);
+               last_was_NT_or_TL=1;
+            }
+            break;
       }
       switch (err) {
          case Err_scan: fprintf(stderr,"Error in parameters: %s\n",s); break;
@@ -592,9 +676,11 @@ void read_nec_input(FILE *f)               /* read the input file, and update th
          case Err_nsurfaces: fprintf(stderr,"Too many surfaces.\n"); break;
          case Err_nexcis: fprintf(stderr,"Too many excitations.\n"); break;
          case Err_nloads: fprintf(stderr,"Too many loads.\n"); break;
+         case Err_nnetws: fprintf(stderr,"Too many networks and/or transmission lines.\n"); break;
       }
       fgets(s,100,f);
    }
+   extr = extr_str;
 }
 
 
